#include <panel.h>
#include <form.h>
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

#include "traindata.h"
#include "accdata.h"
#include "traindialog.h"
#include "configdialog.h"
#include "connection.h"

#define DEBUG_MODE	0

#define APP_MAIN	0
#define APP_ADD		1
#define APP_EDIT	2
#define APP_CONFIG	3


WINDOW 	*cWin;
PANEL	*top, *topA, *console;
struct ctext {
	int col,style;
	char text[128];
} consoleText[CONSOLE_HEIGHT];
int		curTCount, curACount, currentAcc, currentMode, currentTrain;
int		wincols, winrows;

tData*	trains[MAX_TRAINS];
aData*	accs[MAX_ACCS];

trainDialog*	tDialog;
connection*	tConnection;
configDialog*	cDialog;

char	serverName[128];
char	serverPort[10];
char	connectOnStartup[10];

//magic numbers... will be define'd shortly.
int lines=8,cols=40,y=2,x=4,i;

void addConsoleMessage(int col, int style,char* fmt,...) {
	wclear(panel_window(console));
	box(panel_window(console),0,0);
	wbkgd(panel_window(console),COLOR_PAIR(4));
	mvwprintw(panel_window(console),0,1,"[console]");
	int c;
	for (c = 0; c < (CONSOLE_HEIGHT-1); c++) {
		sprintf(consoleText[c].text,"%s",consoleText[c+1].text);
		consoleText[c].col = consoleText[c+1].col;
		consoleText[c].style = consoleText[c+1].style;
		wattron(panel_window(console),COLOR_PAIR(consoleText[c].col) | consoleText[c].style);
		mvwprintw(panel_window(console),c+1,3,"%s",consoleText[c].text);
		wattroff(panel_window(console),COLOR_PAIR(consoleText[c].col) | consoleText[c].style);
	}
	va_list args;
	va_start(args,fmt);
	vsprintf(consoleText[CONSOLE_HEIGHT-1].text,fmt,args);
	va_end(args);
	consoleText[CONSOLE_HEIGHT-1].col = col;
	consoleText[CONSOLE_HEIGHT-1].style = style;
	wattron(panel_window(console),COLOR_PAIR(col) | consoleText[c].style);
	mvwprintw(panel_window(console),CONSOLE_HEIGHT,3,"%s",consoleText[CONSOLE_HEIGHT-1].text);
	wattroff(panel_window(console),COLOR_PAIR(col) | consoleText[c].style);
}

void updatePanelTabbing() {
	top = NULL;
	topA = NULL;
	PANEL *last = NULL;
	for (int p = 0; p < MAX_TRAINS; p++) {
		if (trains[p] != NULL) {
			if (top == NULL) {
				top = trains[p]->tp;
				last = top;
			} else {
				set_panel_userptr(last, trains[p]->tp);
				last = trains[p]->tp;
			}
		}
	}
	if (top != NULL && last != NULL && last != top) {
		set_panel_userptr(last, top);
	}
	last = NULL;
	for (int p = 0; p < MAX_ACCS; p++) {
		if (accs[p] != NULL) {
			if (topA == NULL) {
				topA = accs[p]->tp;
				last = topA;
			} else {
				set_panel_userptr(last, accs[p]->tp);
				last = accs[p]->tp;
			}
		}
	}
	if (topA != NULL && last != NULL && last != topA) {
		set_panel_userptr(last, topA);
	}
}

/*void updateServerMessages() {
	//dump messages from server:
	if (tConnection != NULL) {
		char* mess = tConnection->getLastMessage();
		while (mess != NULL) {
			addConsoleMessage(1,A_NORMAL,"Server: %s",mess);
			mess = tConnection->getLastMessage();
		}
	}
}*/

void initTrain(int idx) {
	if (tConnection != NULL) {
		char* msg = tConnection->sendMessage("INIT 1 GL %d N 1 %d %d",trains[idx]->addr,trains[idx]->mspeed,trains[idx]->numf);
		if (DEBUG_MODE) addConsoleMessage(1,A_BOLD,"INIT 1 GL %d N 1 %d %d [%s]",trains[idx]->addr,trains[idx]->mspeed,trains[idx]->numf,strsep(&msg,"\n\0"));
	}
}

void sendTrainData(int idx) {
	if (tConnection != NULL) {
		char* msg = tConnection->sendMessage("SET 1 GL %d %d %d %d %s",trains[idx]->addr,trains[idx]->dir,trains[idx]->speed,trains[idx]->mspeed,trains[idx]->fStr);
		if (DEBUG_MODE) addConsoleMessage(1,A_BOLD,"SET 1 GL %d %d %d %d %s [%s]",trains[idx]->addr,trains[idx]->dir,trains[idx]->speed,trains[idx]->mspeed,trains[idx]->fStr,strsep(&msg,"\n\0"));
	}
}

void initAcc(int idx) {
	if (tConnection != NULL) {
		char* msg = tConnection->sendMessage("INIT 1 GA %d N",accs[idx]->addr);
		if (DEBUG_MODE) addConsoleMessage(1,A_BOLD,"INIT 1 GL %d N [%s]",accs[idx]->addr,strsep(&msg,"\n\0"));
	}
}

void sendAccData(int idx) {
	if (tConnection != NULL) {
		char* msg = tConnection->sendMessage("SET 1 GA %d 0 %d 0",accs[idx]->addr,accs[idx]->value);
		addConsoleMessage(1,A_BOLD,"SET 1 GA %d 0 %d 0 [%s]",accs[idx]->addr,accs[idx]->value,strsep(&msg,"\n\0"));
	}
}

void connectToServer() {
	tConnection = new connection(serverName,atoi(serverPort));
	if (tConnection->isConnected()) {
		addConsoleMessage(1,A_UNDERLINE,"Successfully connected to server...");
		char* msg = tConnection->sendMessage("SET CONNECTIONMODE SRCP COMMAND");
		addConsoleMessage(1,A_NORMAL,"SET CONNETIONMODE SRCP COMMAND [%s]",strsep(&msg,"\n\0"));
		msg = tConnection->sendMessage("GO");
		addConsoleMessage(1,A_NORMAL,"GO [%s]",strsep(&msg,"\n\0"));
		for (i=0;i<MAX_TRAINS;i++) if (trains[i] != NULL) initTrain(i);
		for (i=0;i<MAX_ACCS;i++) if (accs[i] != NULL) initAcc(i);
	} else addConsoleMessage(2,A_BOLD,"Connection failed!");

//	updateServerMessages();
}

bool createTrain(char* tName,int addr, int mspeed,int nfuncs) {
	if (curTCount + 1 >= MAX_TRAINS) return false;
	bool added = false;
	int i = 0;
	do {
		if (trains[i] == NULL && !added) {
			trains[i] = new tData(tName,addr,mspeed,nfuncs,curTCount);
			updatePanelTabbing();
			currentTrain = i;
			top = trains[currentTrain]->tp;
			curTCount++;
			added = true;
		} else if (trains[i] != NULL) trains[i]->setHighlight(false);
		i++;
	} while (i < MAX_TRAINS);
	return added;
}

bool createAcc(char* aName,int addr,int val) {
	if (curACount + 1 >= MAX_ACCS) return false;
	bool added = false;
	int i = 0;
	do {
		if (accs[i] == NULL && !added) {
			accs[i] = new aData(aName,addr,val,curACount);
			updatePanelTabbing();
			currentAcc = i;
			topA = accs[currentAcc]->tp;
			curACount++;
			added = true;
		} else if (accs[i] != NULL) accs[i]->setHighlight(false);
		i++;
	} while (i < MAX_ACCS);
	return added;
}

void saveConfig() {
	strcpy(serverName,cDialog->getFieldValue(0));
	strcpy(serverPort,cDialog->getFieldValue(1));
	strcpy(connectOnStartup,cDialog->getFieldValue(2));
	addConsoleMessage(3,A_BOLD,"Changes will be saved to disk on exit. [Press F12 (connect/disconnect) to apply changes now.]");
	delete cDialog;
	cDialog = NULL;
	currentMode = APP_MAIN;
}

bool handleInput(int ch) {
	int i;
	switch(currentMode) {
		case APP_MAIN:
			switch(ch) {
				case KEY_F(12):
					if (tConnection == NULL) connectToServer();
					else {
						delete tConnection;
						tConnection = NULL;
						addConsoleMessage(2,A_BOLD,"The connection to the server was terminated");
					}
					break;
				case KEY_F(2):
					if (cDialog == NULL) {
						currentMode = APP_CONFIG;
						cDialog = new configDialog(serverName,serverPort,connectOnStartup);
					}				
					break;
				case KEY_DOWN:
					trains[currentTrain]->changeSpeed(-1);
					sendTrainData(currentTrain);
					break;
				case KEY_UP:
					trains[currentTrain]->changeSpeed(1);
					sendTrainData(currentTrain);
					break;
				case KEY_RIGHT:
					for (i = 0; i < MAX_ACCS; i++) if (accs[i] != NULL) accs[i]->setHighlight(false);
					topA = (PANEL *)panel_userptr(topA);
					top_panel(topA);
					for (i = 0; i < MAX_ACCS; i++) if (accs[i] != NULL && accs[i]->tp == topA) {
						currentAcc = i;
						accs[i]->setHighlight(true);
					}
					break;
				case 10:
					if (accs[currentAcc] != NULL) accs[currentAcc]->toggle();
					break;
				case 338:
					trains[currentTrain]->changeSpeed(-25);
					sendTrainData(currentTrain);
					break;
				case 339:
					trains[currentTrain]->changeSpeed(25);
					sendTrainData(currentTrain);
					break;
				case 9:
					for (i = 0; i < MAX_TRAINS; i++) if (trains[i] != NULL) trains[i]->setHighlight(false);
					top = (PANEL *)panel_userptr(top);
					top_panel(top);
					for (i = 0; i < MAX_TRAINS; i++) if (trains[i] != NULL && trains[i]->tp == top) {
						currentTrain = i;
						trains[i]->setHighlight(true);
					}
					break;
				case 44:
					trains[currentTrain]->changeDirection(1);
					sendTrainData(currentTrain);
					break;
				case 46:
					trains[currentTrain]->changeDirection(0);
					sendTrainData(currentTrain);
					break;
				case 49:
				case 50:
				case 51:
				case 52:
				case 53:
				case 54:
				case 55:
				case 56:
				case 57:
					trains[currentTrain]->toggleFunction(ch-49);
					sendTrainData(currentTrain);
					break;
				case 99:
					if (tDialog == NULL) {
						currentMode = APP_ADD;
						tDialog = new trainDialog(NULL);
					}
					break;
				case 100:
					delete trains[currentTrain];
					trains[currentTrain] = NULL;
					curTCount--;
					updatePanelTabbing();
					for (i = 0; i < MAX_TRAINS; i++) {
						if (trains[i] != NULL) {
							currentTrain = i;
							top_panel(trains[currentTrain]->tp);
							top = trains[currentTrain]->tp;
							continue;
						}
					}
					if (trains[currentTrain] != NULL) trains[currentTrain]->setHighlight(true);
					break;
				case 101:	
					if (tDialog == NULL) {
						currentMode = APP_ADD;
						tDialog = new trainDialog(trains[currentTrain]);
					}
					break;
				case 113:
					return true; //quit
				default:
					addConsoleMessage(1,A_NORMAL,"keypress: %d",ch);
					break;
			}
			break;
		case APP_ADD:
			switch (ch) {
				case 27:
					currentMode = APP_MAIN;
					delete tDialog;
					tDialog = NULL;
					break;
				default:
					if (tDialog->parseInput(ch)) {
						currentMode = APP_MAIN;	
						if (tDialog->t == NULL) createTrain(tDialog->getFieldValue(0),atoi(tDialog->getFieldValue(1)),atoi(tDialog->getFieldValue(2)),atoi(tDialog->getFieldValue(3)));
						trains[currentTrain]->redrawWindow();
						initTrain(currentTrain);
						sendTrainData(currentTrain);
						delete tDialog;
						tDialog = NULL;
					}
					break;
			}
			break;
		case APP_CONFIG:
			switch (ch) {
				case 27:
					delete cDialog;
					cDialog = NULL;
					currentMode = APP_MAIN;
					break;
				default:
					if (cDialog->parseInput(ch)) saveConfig();
					break;
			}
			break;
	}	
	return false;
}

void checkWindowSize() {
	int x,y;
	getmaxyx(stdscr,y,x);
	if (winrows != y || wincols != x) {
		//resize everything.
		WINDOW *old = panel_window(console);
		cWin = newwin(CONSOLE_HEIGHT+2,x,y-CONSOLE_HEIGHT-2,0);	
		replace_panel(console,cWin);
		delwin(old);
		winrows = y;
		wincols = x;
		for (int i =0;i<MAX_TRAINS;i++) if (trains[i] != NULL) trains[i]->resizeWindow(y,x,i);
		//addConsoleMessage(1,A_NORMAL,"new size %d %d old size %d %d",y,x,winrows,wincols);
	}
}

//MEVENT event;

int main() {
	ESCDELAY=0;

	tConnection = NULL;
	tDialog = NULL;
	cDialog = NULL;

	curTCount = 0;
	currentMode = APP_MAIN;

	//write all trains as not-loaded.
	for(i=0;i<MAX_TRAINS;i++) trains[i] = NULL;
	for(i=0;i<MAX_ACCS;i++) accs[i] = NULL;

	initscr();
	start_color();
	cbreak();
	noecho();
	keypad(stdscr, TRUE);
	getmaxyx(stdscr,winrows,wincols);
	
	//mousemask(ALL_MOUSE_EVENTS,NULL);

	init_pair(1,COLOR_WHITE, COLOR_BLUE);
	init_pair(2,COLOR_WHITE, COLOR_RED);
	init_pair(3,COLOR_YELLOW,COLOR_BLACK);
	init_pair(4,COLOR_YELLOW,COLOR_BLUE);
	init_pair(5,COLOR_WHITE, COLOR_GREEN);
	init_pair(6,COLOR_WHITE, COLOR_YELLOW);
	init_pair(7,COLOR_WHITE, COLOR_BLACK);
	init_pair(8,COLOR_WHITE, COLOR_CYAN);
	wbkgd(stdscr,COLOR_PAIR(3));


	console = new_panel(newwin(CONSOLE_HEIGHT+2,wincols,winrows-CONSOLE_HEIGHT-2,0));
	addConsoleMessage(1,A_UNDERLINE,"Welcome to trainControl v%f ... please wait ...",0.1f);

	//--------------load configuration!-----
	FILE *file = fopen("tc.cfg","r");
	if (file != NULL) {
		fscanf(file,"%s\n%s\n%s\n",serverName,serverPort,connectOnStartup);
		addConsoleMessage(1,A_NORMAL,"Loaded settings: %s:%s [Connect on Startup:%s]",serverName,serverPort,connectOnStartup);
		
		char name[32];
		int addr,mspd,nfunc,val;
		//load trains.
		char code[16];
		while (!feof(file)) {
			printf("testing for train.\n");
			fscanf(file,"%s\n",code);
			if (strcmp(code,"GL")==0) {
				fscanf(file,"%s\n",name);
				fscanf(file,"%d %d %d\n",&addr,&mspd,&nfunc);
				createTrain(name,addr,mspd,nfunc);
			} else if (strcmp(code,"GA")==0) {
				fscanf(file,"%s\n",name);
				//addConsoleMessage(1,A_NORMAL,"Acc:%s",name);
				fscanf(file,"%d %d\n",&addr,&val);
				createAcc(name,addr,val);
			} else {
				addConsoleMessage(2,A_BOLD,"FAILED PARSING CONFIG ON CODE: %s",code);
				continue;
			}
		}
		
		fclose(file);
	} else addConsoleMessage(3,A_NORMAL,"Configuration file not found.");
	//---------------------------------------

	update_panels();
	doupdate();

	if (strcmp(connectOnStartup,"on") == 0) connectToServer();
	else if (strlen(serverName) == 0 || strlen(serverPort) == 0) addConsoleMessage(2,A_BOLD,"Server settings missing. Hit F2 to configure!");
	else addConsoleMessage(4,A_NORMAL,"Hit F12 to connect to the server or F2 to change the settings.");

	update_panels();
	doupdate();

	int ch;
	bool quit=false;
	while (!quit && ((ch = getch()) != KEY_F(1))) {
		quit = handleInput(ch);
		checkWindowSize();
		update_panels();
		doupdate();
	}
	
	if (cDialog != NULL) delete cDialog;
	if (tDialog != NULL) delete tDialog;
	if (tConnection != NULL) delete tConnection;

	endwin();
	
	//write settings to disk.
	if (strlen(serverName) > 0 && strlen(serverPort) > 0 && strlen(connectOnStartup) > 0) {
		file = fopen("tc.cfg","w+");
		if (file != NULL) {
			fprintf(file,"%s\n%s\n%s\n",serverName,serverPort,connectOnStartup);
			for (i = 0; i < MAX_TRAINS; i++) {
				if (trains[i] != NULL) {
					fprintf(file,"GL\n%s\n%d %d %d\n",trains[i]->name,trains[i]->addr,trains[i]->mspeed,trains[i]->numf);		
					delete trains[i];		
				}
			}
			for (i = 0; i < MAX_ACCS; i++) {
				if (accs[i] != NULL) {
					fprintf(file,"GA\n%s\n%d %d\n",accs[i]->name,accs[i]->addr,accs[i]->value);		
					delete accs[i];		
				}
			}
			fclose(file);
		}
	} else printf("Error: There were no settings to save!\n");
}
