#include <string.h>
#include "traindata.h"


tData::tData(char* _name, int _addr, int _mspd, int _numf, int offset) {
	sprintf(name,"%s",_name);
	int x,y;
	getmaxyx(stdscr,y,x);
	throttleEnd = y-CONSOLE_HEIGHT-(ACC_OFFSET)-3;
	throttleStart = 2;
	tp = new_panel(newwin(y-CONSOLE_HEIGHT-ACC_OFFSET,TRAIN_WINDOW_COLS,0,offset*TRAIN_WINDOW_COLS));
	wbkgd(panel_window(tp),COLOR_PAIR(1));
	addr = _addr;
	mspeed = _mspd;
	setNFuncs(_numf);
	highlight = true;
	redrawWindow();
}

tData::~tData() {
	if (tp != NULL) {
		//delwin(panel_window(tp));
		del_panel(tp);
	}
}

void tData::resizeWindow(int y, int x,int offset) {
	WINDOW *old = panel_window(tp);
	throttleEnd = y-CONSOLE_HEIGHT-ACC_OFFSET;
	replace_panel(tp,newwin(y-CONSOLE_HEIGHT-ACC_OFFSET,TRAIN_WINDOW_COLS,0,offset*TRAIN_WINDOW_COLS));
	delwin(old);
	redrawWindow();
}

void tData::redrawWindow() {
	wclear(panel_window(tp));
	if (highlight) {
		box(panel_window(tp),0,0);
		mvwprintw(panel_window(tp),0,0,"[%s]",name);
		wbkgd(panel_window(tp),COLOR_PAIR(1));
	} else {
		box(panel_window(tp),0,0);
		mvwprintw(panel_window(tp),0,0,"[%s]",name);
		wbkgd(panel_window(tp),COLOR_PAIR(7));
	}
	mvwprintw(panel_window(tp),1,1,"Adr: %d",addr);
	mvwprintw(panel_window(tp),2,1,"Spd: %d",speed);
	mvwprintw(panel_window(tp),3,1,"Dir: %d",dir);
	mvwprintw(panel_window(tp),4,1,"Fnc:");
	for (int i = 0; i < MAX_FUNCTIONS; i++) {
		if (functions[i] != -1) {
			if (functions[i] == 1) {
				wattron(panel_window(tp),COLOR_PAIR(5));
				mvwprintw(panel_window(tp),4+i,5,"[x]");
				wattroff(panel_window(tp),COLOR_PAIR(5));
			}
			else mvwprintw(panel_window(tp),4+i,5,"[_]");
		}
	}
	int tpos = throttleEnd - (((((speed)*100)/(mspeed)  * (throttleEnd - throttleStart)) / 100)),col;
	for (int i=throttleEnd; i >= throttleStart; i--) {
		if (i > (throttleEnd - throttleStart) - ((throttleEnd - throttleStart) / 3)) col = 5;
		else if (i > (throttleEnd - throttleStart) / 3) col = 6;
		else col = 2;
		wattron(panel_window(tp),COLOR_PAIR(col));
		wmove(panel_window(tp),i,10);
		if (tpos != i) waddch(panel_window(tp),'|' | A_BOLD);
		else {
			mvwprintw(panel_window(tp),i,9,"-+-");
		}
		wattroff(panel_window(tp),COLOR_PAIR(col));
	}
}

void tData::setHighlight(bool h) {
	highlight = h;
	redrawWindow();
}

void tData::toggleFunction(int f) {
	if (f < numf) {
		functions[f] += 1;
		if (functions[f] > 1) functions[f] = 0;
		sprintf(fStr,"");
		for (int i = 0; i < numf; i++) sprintf(fStr,"%s %d",fStr,functions[i]);
		redrawWindow();
	}
}

void tData::setNFuncs(int nf) {
	numf = nf;
	resetFuncs();
}

void tData::resetFuncs() {
	for (int f = 0; f < MAX_FUNCTIONS; f++) {
		if (f < numf) functions[f]=0;
		else functions[f] = -1;
	}
	sprintf(fStr,"");
	for (int i = 0; i < numf; i++) sprintf(fStr,"%s %d",fStr,functions[i]);
}

void tData::setSpeed(int spd) {
	speed = spd;
	redrawWindow();
}
	
void tData::changeSpeed(int add) {
	speed += add;
	if (speed < 0) speed = 0;
	if (speed > mspeed) speed = mspeed;
	redrawWindow();
}
void tData::changeDirection(int _dir) {
	dir=_dir;
	redrawWindow();
}
void tData::changeAddress(int _addr) {
	addr = _addr;
	redrawWindow();
}


//helper functions
char *strip(char *haystack, char *needles) {
        char *found;
        while ((found = strpbrk(haystack,needles)) != NULL) strcpy(found,found+1);
        return haystack;
}


